@@ -251,6 +251,8 @@ SvnHelper::SvnHelper():KApplication() {
 		KIO::SimpleJob * job = KIO::special(servURL, parms);
 		connect( job, SIGNAL( result( KJob * ) ), this, SLOT( slotResult( KJob * ) ) );
 		KIO::NetAccess::synchronousRun( job, 0 );
+	} else if (args->isSet("R")) {
+		kDebug(7128) << "rename/move TODO " << list;
 	} else if (args->isSet("C")) {
 		kDebug(7128) << "checkout " << list;
 		SubversionCheckoutDialog d;
