@@ -1,13 +1,11 @@
 /***************************************************************************
-                          kompareconnectwidget.h  -  description
-                             -------------------
+                          kompareconnectwidget.h
+                          ----------------------
     begin                : Tue Jun 26 2001
-    copyright            : (C) 2001-2003 John Firebaugh
-                           (C) 2001-2004 Otto Bruggeman
-                           (C) 2004      Jeff Snyder
-    email                : jfirebaugh@kde.org
-                           otto.bruggeman@home.nl
-                           jeff@caffeinated.me.uk
+    Copyright 2001-2003 John Firebaugh <jfirebaugh@kde.org>
+    Copyright 2001-2004 Otto Bruggeman <otto.bruggeman@home.nl>
+    Copyright 2004      Jeff Snyder    <jeff@caffeinated.me.uk>
+    Copyright 2007      Kevin Kofler   <kevin.kofler@chello.at>
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,8 +20,15 @@
 #ifndef KOMPARECONNECTWIDGET_H
 #define KOMPARECONNECTWIDGET_H
 
-#include "kompare_qsplitter.h"
-#include <qwidget.h>
+#include <QtGui/QWidget>
+#include <QtGui/QSplitter>
+#include <Q3PointArray>
+#include <QtGui/QPaintEvent>
+#include <QtGui/QMouseEvent>
+#include <Q3VBoxLayout>
+#include <QtGui/QLabel>
+
+#include "komparemodellist.h"
 
 namespace Diff2 {
 class DiffModel;
@@ -37,8 +42,7 @@ class KompareConnectWidget : public QWidget
 	Q_OBJECT
 
 public:
-	KompareConnectWidget( KompareListView* left, KompareListView* right,
-	      ViewSettings* settings, QWidget* parent, const char* name = 0 );
+	KompareConnectWidget( ViewSettings* settings, QWidget* parent, const char* name = 0 );
 	~KompareConnectWidget();
 
 public slots:
@@ -52,16 +56,13 @@ signals:
 
 protected:
 	void paintEvent( QPaintEvent* e );
-	QPointArray makeTopBezier( int tl, int tr );
-	QPointArray makeBottomBezier( int bl, int br );
-	QPointArray makeConnectPoly( const QPointArray& topBezier, const QPointArray& bottomBezier );
+	Q3PointArray makeTopBezier( int tl, int tr );
+	Q3PointArray makeBottomBezier( int bl, int br );
+	Q3PointArray makeConnectPoly( const Q3PointArray& topBezier, const Q3PointArray& bottomBezier );
 
 private:
 	ViewSettings*             m_settings;
 
-	KompareListView*          m_leftView;
-	KompareListView*          m_rightView;
-
 	const Diff2::DiffModel*   m_selectedModel;
 	const Diff2::Difference*  m_selectedDifference;
 };
@@ -70,8 +71,7 @@ class KompareConnectWidgetFrame : public QSplitterHandle
 {
 	Q_OBJECT
 public:
-	KompareConnectWidgetFrame( KompareListView* left, KompareListView* right,
-	      ViewSettings* settings, KompareSplitter* parent, const char* name = 0 );
+	KompareConnectWidgetFrame( ViewSettings* settings, KompareSplitter* parent, const char* name = 0 );
 	~KompareConnectWidgetFrame();
 	
 	QSize sizeHint() const;
@@ -82,10 +82,16 @@ protected:
 	// stop the parent QSplitterHandle painting
 	void paintEvent( QPaintEvent* /* e */ ) { }
 
+#if 0
+	void mouseMoveEvent( QMouseEvent * );
+	void mousePressEvent( QMouseEvent * );
+	void mouseReleaseEvent( QMouseEvent * );
+#endif
+
 private:
 	KompareConnectWidget      m_wid;
 	QLabel                    m_label;
-	QVBoxLayout               m_layout;
+	Q3VBoxLayout               m_layout;
 };
 
 #endif
