@@ -19,13 +19,13 @@
 #define PROPSVIEW_H
 
 #include <qwidget.h>
-#include <qlistview.h>
+#include <k3listview.h>
 
 /**
   *@author Richard Moore
   */
 
-class PropsView : public QListView  {
+class PropsView : public K3ListView  {
    Q_OBJECT
 public: 
 	PropsView(QWidget *parent=0, const char *name=0);
