@@ -1,5 +1,5 @@
 /* ****************************************************************************
-  This file is part of KAider
+  This file is part of Lokalize
 
   Copyright (C) 2007 by Nick Shaforostoff <shafff@ukr.net>
 
@@ -29,7 +29,7 @@
   your version.
 
 **************************************************************************** */
-#define KDE_NO_DEBUG_OUTPUT
+//#define KDE_NO_DEBUG_OUTPUT
 #include "projectmodel.h"
 #include "project.h"
 
@@ -40,16 +40,18 @@
 #include <QTime>
 #include <QFile>
 
+#undef KDE_NO_DEBUG_OUTPUT
 
-ProjectModel::ProjectModel()
-    : KDirModel()
+
+ProjectModel::ProjectModel(QObject *parent)
+    : KDirModel(parent)
     , m_dirIcon(KIcon(QLatin1String("inode-directory")))
     , m_poIcon(KIcon(QLatin1String("flag-blue")))
     , m_poComplIcon(KIcon(QLatin1String("flag-green")))
     , m_potIcon(KIcon(QLatin1String("flag-black")))
 {
-    connect (this,SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&) ),
-             this,SLOT(aa()));
+//     connect (this,SIGNAL(dataChanged(const QModelIndex&,const QModelIndex&) ),
+//              this,SLOT(aa()));
 
     setDirLister(new ProjectLister(this));
 }
@@ -61,7 +63,7 @@ ProjectModel::ProjectModel()
  * order is tran,  untr, fuzzy
  *          left() top() width()
  *
- * 32 is a secret code that we use to say that info isnot ready yet
+ * 32,64 is a secret code that we use to say that info is not ready yet
  */
 QVariant ProjectModel::data ( const QModelIndex& index, int role) const
 {
@@ -97,7 +99,7 @@ QVariant ProjectModel::data ( const QModelIndex& index, int role) const
             int translated=0;
             int fuzzy=0;
 
-//takes 5 mseconds on kdebase
+//takes 5 mseconds on messages/kdebase
 #if 0
             //first, check if we have already calcalated real stats
             if (!metaInfo.item("translation.translated").value().isNull())
@@ -146,21 +148,25 @@ QVariant ProjectModel::data ( const QModelIndex& index, int role) const
                 }
             }
 
-            if (infoIsFull&&(untranslated+translated+fuzzy))
+            //if (/*infoIsFull&&*/(untranslated+translated+fuzzy))
+            if (infoIsFull)
             {
 
 //                 KFileMetaInfo dirInfo(item->metaInfo(false));
 #if 1
-                metaInfo.item("translation.untranslated").setValue(untranslated);
-                metaInfo.item("translation.translated").setValue(translated);
-                metaInfo.item("translation.fuzzy").setValue(fuzzy);
-                item.setMetaInfo(metaInfo);
+                if (infoIsFull)
+                {
+                    metaInfo.item("translation.untranslated").setValue(untranslated);
+                    metaInfo.item("translation.translated").setValue(translated);
+                    metaInfo.item("translation.fuzzy").setValue(fuzzy);
+                    item.setMetaInfo(metaInfo);
+                }
 #endif
 
                 switch(column)
                 {
                     case Graph:
-                        return QRect(translated,untranslated,fuzzy,0);
+                        return QRect(translated,untranslated,fuzzy,infoIsFull?0:64);
                     case Total:
                         return translated+untranslated+fuzzy;
                     case Translated:
@@ -227,6 +233,26 @@ QVariant ProjectModel::data ( const QModelIndex& index, int role) const
     }
     return KDirModel::data(index,role);
 }
+/*
+void ProjectModel::readRecursively(const QModelIndex& index)
+{
+    QCoreApplication::processEvents(QEventLoop::AllEvents);
+    data(index);
+    int count=rowCount(index);
+    kWarning()<<"dddd"<<count;
+    int i=0;
+    bool infoIsFull=true;
+    //QTime a;a.start();
+    for (;i<count;++i)
+    {
+        if (hasChildren(index.child(i,0))&&!data(index.child(i,0)).isNull())
+            readRecursively(index.child(i,0));
+        else
+            data(index.child(i,Graph));
+    }
+
+}*/
+
 
 QVariant ProjectModel::headerData(int section, Qt::Orientation orientation, int role) const
 {
@@ -258,22 +284,24 @@ QVariant ProjectModel::headerData(int section, Qt::Orientation orientation, int
 }
 
 
+void ProjectModel::openUrl(const KUrl& u)
+{
+    //kDebug()<<"called";
+    kWarning()<<"called";
+    static_cast<ProjectLister*>(dirLister())->cleanup();
+    dirLister()->openUrl(u);
+}
+
+
 
 ProjectLister::ProjectLister(ProjectModel* model, QObject *parent)
     : KDirLister(parent)
-    , m_templates(new KDirLister (this))
+    , m_templates(0)
     , m_reactOnSignals(true)
     , m_model(model)
 {
-    connect(m_templates,SIGNAL(newItems(KFileItemList)),
-            this, SLOT(slotNewTemplItems(KFileItemList)));
-    connect(m_templates, SIGNAL(deleteItem(const KFileItem&)),
-            this, SLOT(slotDeleteTemplItem(const KFileItem&)));
-    connect(m_templates, SIGNAL(refreshItems(QList< QPair< KFileItem, KFileItem > > )),
-            this, SLOT(slotRefreshTemplItems(QList< QPair< KFileItem, KFileItem > > )));
+    cleanup();//setups m_templates
 
-    m_templates->setNameFilter("*.pot");
-    setNameFilter("*.po *.pot");
 //         connect( m_templates, SIGNAL(clear()),
 //                 this, SLOT(slotClear()) );
     //NOTE ??
@@ -312,7 +340,35 @@ bool ProjectLister::openUrlRecursive(const KUrl &_url, KDirLister::OpenUrlFlags
     return openUrl(_url, _flags);
 }
 
-/* doesnt handle .po to .po_t_ :) */
+
+void ProjectLister::cleanup()
+{
+    if (m_templates)
+        m_templates->deleteLater();
+    m_templates = new KDirLister(this);
+
+    connect(m_templates,SIGNAL(newItems(KFileItemList)),
+            this, SLOT(slotNewTemplItems(KFileItemList)));
+    connect(m_templates, SIGNAL(deleteItem(const KFileItem&)),
+            this, SLOT(slotDeleteTemplItem(const KFileItem&)));
+    connect(m_templates, SIGNAL(refreshItems(QList< QPair< KFileItem, KFileItem > > )),
+            this, SLOT(slotRefreshTemplItems(QList< QPair< KFileItem, KFileItem > > )));
+
+    m_templates->setNameFilter("*.pot");
+    setNameFilter("*.po *.pot *.xlf");
+
+
+
+
+
+    m_hiddenTemplItems.clear();
+    m_recursiveUrls.clear();
+    m_listedTemplDirs.clear();
+    //m_reactOnSignals=true;
+}
+
+
+/* does not handle .po to .po_t_ :) */
 static bool poToPot(QString& path)
 {
     if (path.isEmpty()||Project::instance()->poDir().isEmpty()||Project::instance()->potDir().isEmpty())
@@ -322,7 +378,7 @@ static bool poToPot(QString& path)
     return true;
 }
 
-/* doesnt handle .po to .po_t_ */
+/* does not handle .po to .po_t_ */
 static bool potToPo(QString& path)
 {
     if (path.isEmpty()||Project::instance()->poDir().isEmpty()||Project::instance()->potDir().isEmpty())
@@ -357,7 +413,8 @@ void ProjectLister::slotCompleted(const KUrl& _url)
         {
             if (list.at(i).isDir())
             {
-                kapp->processEvents();
+                QCoreApplication::processEvents();
+                QCoreApplication::processEvents(QEventLoop::WaitForMoreEvents,200);
                 openUrlRecursive(list.at(i).url());
             }
         }
@@ -570,7 +627,7 @@ void ProjectLister::slotDeleteItem(const KFileItem& item)
             KFileItemList list;
             list.append(po);
             kDebug()<<"emitting templ:"<<poPath;
-            //HACK for debuging
+            //HACK for debugging
             if (m_model->indexForUrl(KUrl(  po.url().directory()  )).isValid())
                 emit newItems(list);
             else
@@ -779,3 +836,5 @@ void ProjectLister::slotFileSaved(const KUrl& url)
 
 
 
+
+#include "projectmodel.moc"
