@@ -34,13 +34,18 @@
 class ActivityWidget : public UMLWidget {
     Q_OBJECT
 
+    QString preText;
+    QString postText;
 public:
     enum ActivityType
     {
         Initial = 0,
         Normal,
         End,
-        Branch
+        Final,
+        Branch,
+        Invok,
+        Param
     };
 
     /**
@@ -105,6 +110,27 @@ public:
      */
     bool loadFromXMI( QDomElement & qElement );
 
+
+    /**
+     * This method set the name of the preText attribute
+     */
+     void setPreText(const QString&);
+
+    /**
+     * This method get the name of the preText attribute
+     */
+     QString getPreText();
+
+     /**
+     * This method set the name of the postText attribute
+     */
+     void setPostText(const QString&);
+
+   /**
+     * This method get the name of the postText attribute
+     */
+     QString getPostText();
+
 protected:
     /**
      * Overrides method from UMLWidget
@@ -116,12 +142,19 @@ protected:
      */
     ActivityType m_ActivityType;
 
+
+    /**
+     * Type of normal activity (Invok or not).
+     * This function is call by the dialog box properties
+     */
+    bool m_NormalActivityType;
+
 public slots:
 
     /**
      * Captures any popup menu signals for menus it created.
      */
-    void slotMenuSelection(int sel);
+    void slotMenuSelection(QAction* action);
 };
 
 #endif
