@@ -1,18 +1,17 @@
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2006                                                    *
+ *   copyright (C) 2006-2008                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef CODEGENPOLICYEXT_H
 #define CODEGENPOLICYEXT_H
 
-#include <qobject.h>
+#include <QtCore/QObject>
 
 class QWidget;
 class KConfig;
@@ -26,8 +25,8 @@ class CodeGenerationPolicyPage;
  * @author Oliver Kellogg <okellogg@users.sourceforge.net>
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
-
-class CodeGenPolicyExt : public QObject {
+class CodeGenPolicyExt : public QObject
+{
     Q_OBJECT
 public:
     CodeGenPolicyExt() {}
