@@ -1,30 +1,31 @@
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2006                                               *
+ *   copyright (C) 2002-2008                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
 #ifndef NOTEDIALOG_H
 #define NOTEDIALOG_H
-//kde includes
+
+// app includes
+#include "notewidget.h"
+
+// kde includes
 #include <kdialog.h>
-//app includes
-#include "../notewidget.h"
+
+class KTextEdit;
+class QGroupBox;
 
 /**
  * @author Paul Hensgen
  * Bugs and comments to uml-devel@lists.sf.net or http://bugs.kde.org
  */
-class Q3GroupBox;
-class Q3TextEdit;
-
-class NoteDialog : public KDialog {
-
+class NoteDialog : public KDialog
+{
    Q_OBJECT
 
 public:
@@ -40,10 +41,11 @@ public:
 
 public slots:
     void slotOk();
+
 private:
     //GUI widgets
-    Q3GroupBox * m_pDocGB;
-    Q3TextEdit * m_pDocTE;
+    QGroupBox * m_pDocGB;
+    KTextEdit * m_pDocTE;
 
     /**
      *  Note widget to show documentation for.
