@@ -17,9 +17,9 @@
 #include <cmath>
 
 // qt includes
-#include <qcanvas.h>
-#include <qdatastream.h>
-#include <qdom.h>
+#include <q3canvas.h>
+#include <QDomDocument>
+#include <QPainter>
 
 // kde includes
 #include <kdebug.h>
@@ -32,24 +32,25 @@
 #include "umldoc.h"
 #include "uml.h"
 
-LinePath::Circle::Circle(QCanvas * canvas, int radius /* = 0 */)
-        : QCanvasEllipse(radius * 2, radius * 2, canvas) {
+
+LinePath::Circle::Circle(Q3Canvas * canvas, int radius /* = 0 */)
+        : Q3CanvasEllipse(radius * 2, radius * 2, canvas) {
 }
 
 void LinePath::Circle::setX(int x) {
-    QCanvasItem::setX( (double) x );
+    Q3CanvasItem::setX( (double) x );
 }
 
 void LinePath::Circle::setY(int y) {
-    QCanvasItem::setY( (double) y );
+    Q3CanvasItem::setY( (double) y );
 }
 
 void LinePath::Circle::setRadius(int radius) {
-    QCanvasEllipse::setSize(radius * 2, radius * 2);
+    Q3CanvasEllipse::setSize(radius * 2, radius * 2);
 }
 
 int LinePath::Circle::getRadius() const {
-    return (QCanvasEllipse::height() / 2);
+    return (Q3CanvasEllipse::height() / 2);
 }
 
 void LinePath::Circle::drawShape(QPainter& p) {
@@ -59,22 +60,22 @@ void LinePath::Circle::drawShape(QPainter& p) {
 }
 
 LinePath::LinePath() {
-    m_RectList.setAutoDelete( true );
-    m_LineList.setAutoDelete( true );
-    m_HeadList.setAutoDelete( true );
-    m_ParallelList.setAutoDelete( true );
     m_bSelected = false;
     m_pClearPoly = 0;
     m_pCircle = 0;
+    m_pSubsetSymbol = 0;
     m_PointArray.resize( 4 );
     m_ParallelLines.resize( 4 );
     m_pAssociation = 0;
     m_bHeadCreated = false;
+    m_bSubsetSymbolCreated = false;
     m_bParallelLineCreated = false;
     m_DockRegion = TopBottom;
 }
 
-LinePath::~LinePath() {}
+LinePath::~LinePath()
+{
+}
 
 void LinePath::setAssociation(AssociationWidget * association ) {
     if( !association )
@@ -82,6 +83,7 @@ void LinePath::setAssociation(AssociationWidget * association ) {
     cleanup();
     m_pAssociation = association;
     createHeadLines();
+    createSubsetSymbol();
     if( getAssocType() == Uml::at_Coll_Message )
         setupParallelLine();
     UMLView * view =  (UMLView *)m_pAssociation -> parent();
@@ -89,16 +91,16 @@ void LinePath::setAssociation(AssociationWidget * association ) {
     connect( view, SIGNAL( sigLineWidthChanged( Uml::IDType ) ), this, SLOT( slotLineWidthChanged( Uml::IDType ) ) );
 }
 
-QPoint LinePath::getPoint( int pointIndex ) {
+QPoint LinePath::getPoint( int pointIndex ) const {
     int count = m_LineList.count();
     if( count == 0 || pointIndex > count  || pointIndex < 0)
         return QPoint( -1, -1 );
 
     if( pointIndex == count ) {
-        QCanvasLine * line = m_LineList.last();
+        Q3CanvasLine * line = m_LineList.last();
         return line -> endPoint();
     }
-    QCanvasLine * line = m_LineList.at( pointIndex );
+    Q3CanvasLine * line = m_LineList.at( pointIndex );
     return line -> startPoint();
 }
 
@@ -107,12 +109,12 @@ bool LinePath::setPoint( int pointIndex, const QPoint &point ) {
     if( count == 0 || pointIndex > count  || pointIndex < 0)
         return false;
     if (point.x() == 0 && point.y() == 0) {
-        kError() << "LinePath::setPoint:ignoring request for (0,0)" << endl;
+        uError() << "ignoring request for (0,0)" << endl;
         return false;
     }
 
     if( pointIndex == count) {
-        QCanvasLine * line = m_LineList.last();
+        Q3CanvasLine * line = m_LineList.last();
         QPoint p = line -> startPoint();
         line -> setPoints( p.x(), p.y(), point.x(), point.y() );
         moveSelected( pointIndex );
@@ -120,14 +122,14 @@ bool LinePath::setPoint( int pointIndex, const QPoint &point ) {
         return true;
     }
     if( pointIndex == 0 ) {
-        QCanvasLine * line = m_LineList.first();
+        Q3CanvasLine * line = m_LineList.first();
         QPoint p = line -> endPoint();
         line -> setPoints( point.x(), point.y(), p.x(), p.y() );
         moveSelected( pointIndex );
         update();
         return true;
     }
-    QCanvasLine * line = m_LineList.at( pointIndex  );
+    Q3CanvasLine * line = m_LineList.at( pointIndex  );
     QPoint p = line -> endPoint();
     line -> setPoints( point.x(), point.y(), p.x(), p.y() );
     line = m_LineList.at( pointIndex - 1 );
@@ -143,7 +145,7 @@ bool LinePath::isPoint( int pointIndex, const QPoint &point, unsigned short delt
     if ( pointIndex >= count )
         return false;
 
-    QCanvasLine * line = m_LineList.at( pointIndex );
+    Q3CanvasLine * line = m_LineList.at( pointIndex );
 
     /* check if the given point is the start or end point of the line */
     if ( (
@@ -168,11 +170,11 @@ bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
     const bool bLoading = UMLApp::app()->getDocument()->loading();
 
     if( count == 1 || pointIndex == 1) {
-        QCanvasLine * first = m_LineList.first();
+        Q3CanvasLine * first = m_LineList.first();
         QPoint sp = first -> startPoint();
         QPoint ep = first -> endPoint();
         first -> setPoints( sp.x(), sp.y(), point.x(), point.y() );
-        QCanvasLine * line = new QCanvasLine( getCanvas() );
+        Q3CanvasLine * line = new Q3CanvasLine( getCanvas() );
         line -> setZ( -2 );
         line -> setPoints( point.x(), point.y(), ep.x(), ep.y() );
         line -> setPen( getPen() );
@@ -183,11 +185,11 @@ bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
         return true;
     }
     if( count + 1 == pointIndex ) {
-        QCanvasLine * before = m_LineList.last();
+        Q3CanvasLine * before = m_LineList.last();
         QPoint sp = before -> startPoint();
         QPoint ep = before -> endPoint();
         before -> setPoints( sp.x(), sp.y(), point.x(), point.y() );
-        QCanvasLine * line = new QCanvasLine( getCanvas() );
+        Q3CanvasLine * line = new Q3CanvasLine( getCanvas() );
         line -> setPoints( point.x(), point.y(), ep.x(), ep.y() );
         line -> setZ( -2 );
         line -> setPen( getPen() );
@@ -197,11 +199,11 @@ bool LinePath::insertPoint( int pointIndex, const QPoint &point ) {
             setupSelected();
         return true;
     }
-    QCanvasLine * before = m_LineList.at( pointIndex - 1 );
+    Q3CanvasLine * before = m_LineList.at( pointIndex - 1 );
     QPoint sp = before -> startPoint();
     QPoint ep = before -> endPoint();
     before -> setPoints( sp.x(), sp.y(), point.x(), point.y() );
-    QCanvasLine * line = new QCanvasLine(getCanvas() );
+    Q3CanvasLine * line = new Q3CanvasLine(getCanvas() );
     line -> setPoints( point.x(), point.y(), ep.x(), ep.y() );
     line -> setZ( -2 );
     line -> setPen( getPen() );
@@ -221,7 +223,7 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
 
     /* we don't know if the user clicked on the start- or endpoint of a
      * line segment */
-    QCanvasLine * current_line = m_LineList.at( pointIndex );
+    Q3CanvasLine * current_line = m_LineList.at( pointIndex );
     if (abs( current_line -> endPoint().x() - point.x() ) <= delta
             &&
             abs( current_line -> endPoint().y() - point.y() ) <= delta)
@@ -233,7 +235,7 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
 
         /* the next segment will get the starting point from the current one,
          * which is going to be removed */
-        QCanvasLine * next_line = m_LineList.at( pointIndex + 1 );
+        Q3CanvasLine * next_line = m_LineList.at( pointIndex + 1 );
         QPoint startPoint = current_line -> startPoint();
         QPoint endPoint = next_line -> endPoint();
         next_line -> setPoints(startPoint.x(), startPoint.y(),
@@ -251,7 +253,7 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
 
             /* the previous segment will get the end point from the current one,
              * which is going to be removed */
-            QCanvasLine * previous_line = m_LineList.at( pointIndex - 1 );
+            Q3CanvasLine * previous_line = m_LineList.at( pointIndex - 1 );
             QPoint startPoint = previous_line -> startPoint();
             QPoint endPoint = current_line -> endPoint();
             previous_line -> setPoints(startPoint.x(), startPoint.y(),
@@ -264,17 +266,16 @@ bool LinePath::removePoint( int pointIndex, const QPoint &point, unsigned short
 
 
     /* remove the segment from the list */
-    m_LineList.remove( pointIndex );
+    delete m_LineList.takeAt( pointIndex );
 
     return true;
 }
 
 bool LinePath::setStartEndPoints( const QPoint &start, const QPoint &end ) {
     int count = m_LineList.count();
-
     if( count == 0 ) {
-        QCanvasLine * line = new QCanvasLine(getCanvas() );
-        line -> setPoints( start.x(), start.y(), end.x(), end.y() );
+        Q3CanvasLine * line = new Q3CanvasLine(getCanvas() );
+        line -> setPoints( start.x(), start.y(),end.x(),end.y() );
         line -> setZ( -2 );
         line -> setPen( getPen() );
         line -> setVisible( true );
@@ -283,21 +284,21 @@ bool LinePath::setStartEndPoints( const QPoint &start, const QPoint &end ) {
     }
     bool status = setPoint( 0, start );
     if( status)
-        return setPoint( count , end );
+        return setPoint(count,end);
     return false;
 }
 
-int LinePath::count() {
+int LinePath::count() const {
     return m_LineList.count() + 1;
 }
 
 int LinePath::onLinePath( const QPoint &position ) {
-    QCanvasItemList list = getCanvas() -> collisions( position );
+    Q3CanvasItemList list = getCanvas() -> collisions( position );
     int index = -1;
 
-    QCanvasItemList::iterator end(list.end());
-    for(QCanvasItemList::iterator item_it(list.begin()); item_it != end; ++item_it ) {
-        if( ( index = m_LineList.findRef( (QCanvasLine*)*item_it ) ) != -1 )
+    Q3CanvasItemList::iterator end(list.end());
+    for(Q3CanvasItemList::iterator item_it(list.begin()); item_it != end; ++item_it ) {
+        if( ( index = m_LineList.indexOf( (Q3CanvasLine*)*item_it ) ) != -1 )
             break;
     }//end for
     return index;
@@ -306,25 +307,29 @@ int LinePath::onLinePath( const QPoint &position ) {
 void LinePath::setSelected( bool select ) {
     if(select)
         setupSelected();
-    else if( m_RectList.count() > 0 )
+    else if( m_RectList.count() > 0 ) {
+        qDeleteAll( m_RectList.begin(), m_RectList.end() );
         m_RectList.clear();
+    }
 }
 
 void LinePath::setAssocType( Uml::Association_Type type ) {
-    LineListIt it( m_LineList );
-    QCanvasLine * line = 0;
-    while( ( line = it.current() ) ) {
-        line -> setPen( getPen() );
-        ++it;
-    }
+    QList<Q3CanvasLine*>::Iterator it = m_LineList.begin();
+    QList<Q3CanvasLine*>::Iterator end = m_LineList.end();
+
+    for( ; it != end; ++it )
+        (*it) -> setPen( getPen() );
+
     if( m_pClearPoly ) {
         delete m_pClearPoly;
         m_pClearPoly = 0;
     }
     if( type == Uml::at_Coll_Message )
         setupParallelLine();
-    else
+    else {
         createHeadLines();
+        createSubsetSymbol();
+    }
     update();
 }
 
@@ -341,6 +346,12 @@ void LinePath::update() {
     } else {
         createHeadLines();
     }
+
+    if ( m_bSubsetSymbolCreated ) {
+        updateSubsetSymbol();
+    } else {
+        createSubsetSymbol();
+    }
 }
 
 void LinePath::slotLineColorChanged( Uml::IDType viewID ) {
@@ -352,31 +363,29 @@ void LinePath::slotLineColorChanged( Uml::IDType viewID ) {
 
 
 void LinePath::setLineColor( const QColor &color ) {
-    QCanvasLine * line = 0;
     uint linewidth = 0;
-    LineListIt it( m_LineList );
-    while( ( line = it.current() ) ) {
+    Q3CanvasLine * line = 0;
+
+    Q_FOREACH( line, m_LineList ) {
         linewidth = line->pen().width();
         line -> setPen( QPen( color, linewidth ) );
-        ++it;
-    }
-    LineListIt hit( m_HeadList );
-    while( ( line = hit.current() ) ) {
-        linewidth = line->pen().width();
-        line -> setPen( QPen( color, linewidth ) );
-        ++hit;
-    }
-    LineListIt pit( m_ParallelList );
-    while( ( line = pit.current() ) ) {
-        linewidth = line->pen().width();
-        line -> setPen( QPen( color, linewidth ) );
-        ++pit;
     }
 
-    if( getAssocType() == Uml::at_Aggregation )
+    Q_FOREACH( line, m_HeadList ) {
+        linewidth = line->pen().width();
+        line -> setPen( QPen( color, linewidth ) );
+    }
+
+    Q_FOREACH( line, m_ParallelList ) {
+        linewidth = line->pen().width();
+        line -> setPen( QPen( color, linewidth ) );
+    }
+
+    if (getAssocType() == Uml::at_Aggregation) {
         if (m_pClearPoly) m_pClearPoly -> setBrush( QBrush( Qt::white ) );
         else if( getAssocType() == Uml::at_Composition )
             if (m_pClearPoly) m_pClearPoly -> setBrush( QBrush( color ) );
+    }
 
     if( m_pCircle ) {
         linewidth = m_pCircle->pen().width();
@@ -392,25 +401,22 @@ void LinePath::slotLineWidthChanged( Uml::IDType viewID ) {
 }
 
 void LinePath::setLineWidth( uint width ) {
-    QCanvasLine * line = 0;
     QColor linecolor;
-    LineListIt it( m_LineList );
-    while( ( line = it.current() ) ) {
+    Q3CanvasLine * line = 0;
+
+    Q_FOREACH( line, m_LineList ) {
         linecolor = line->pen().color();
         line -> setPen( QPen( linecolor, width ) );
-        ++it;
     }
-    LineListIt hit( m_HeadList );
-    while( ( line = hit.current() ) ) {
+
+    Q_FOREACH( line, m_HeadList ) {
         linecolor = line->pen().color();
         line -> setPen( QPen( linecolor, width ) );
-        ++hit;
     }
-    LineListIt pit( m_ParallelList );
-    while( ( line = pit.current() ) ) {
+
+    Q_FOREACH( line, m_ParallelList ) {
         linecolor = line->pen().color();
         line -> setPen( QPen( linecolor, width ) );
-        ++pit;
     }
 
     if( m_pCircle ) {
@@ -422,13 +428,14 @@ void LinePath::setLineWidth( uint width ) {
 void LinePath::moveSelected( int pointIndex ) {
     int lineCount = m_LineList.count();
     if( !m_bSelected ) {
+        qDeleteAll( m_RectList.begin(), m_RectList.end() );
         m_RectList.clear();
         return;
     }
     if( (int)m_RectList.count() + 1 != lineCount )
         setupSelected();
-    QCanvasRectangle * rect = 0;
-    QCanvasLine * line = 0;
+    Q3CanvasRectangle * rect = 0;
+    Q3CanvasLine * line = 0;
     if( pointIndex == lineCount || lineCount == 1) {
         line = m_LineList.last();
         QPoint p = line -> endPoint();
@@ -447,19 +454,19 @@ void LinePath::moveSelected( int pointIndex ) {
 }
 
 void LinePath::setupSelected() {
+    qDeleteAll( m_RectList.begin(), m_RectList.end() );
     m_RectList.clear();
-    QCanvasLine * line = 0;
-    LineListIt it( m_LineList );
-    while( ( line = it.current() ) ) {
+    Q3CanvasLine * line = 0;
+
+    Q_FOREACH( line, m_LineList ) {
         QPoint sp = line -> startPoint();
-        QCanvasRectangle *rect = Widget_Utils::decoratePoint(sp);
+        Q3CanvasRectangle *rect = Widget_Utils::decoratePoint(sp);
         m_RectList.append( rect );
-        ++it;
     }
     //special case for last point
     line = m_LineList.last();
     QPoint p = line -> endPoint();
-    QCanvasRectangle *rect = Widget_Utils::decoratePoint(p);
+    Q3CanvasRectangle *rect = Widget_Utils::decoratePoint(p);
     m_RectList.append( rect );
     update();
 }
@@ -555,15 +562,17 @@ void LinePath::calculateHead() {
 
 void LinePath::updateHead() {
     int count = m_HeadList.count();
-    QCanvasLine * line = 0;
+    Q3CanvasLine * line = 0;
 
     switch( getAssocType() ) {
     case Uml::at_State:
     case Uml::at_Activity:
+    case Uml::at_Exception:
     case Uml::at_UniAssociation:
     case Uml::at_Dependency:
         if( count < 2)
             return;
+
         line = m_HeadList.at( 0 );
         line -> setPoints( m_EgdePoint.x(), m_EgdePoint.y(), m_ArrowPointA.x(), m_ArrowPointA.y() );
 
@@ -641,7 +650,7 @@ void LinePath::updateHead() {
 void LinePath::growList(LineList &list, int by) {
     QPen pen( getLineColor(), getLineWidth() );
     for (int i = 0; i < by; i++) {
-        QCanvasLine * line = new QCanvasLine( getCanvas() );
+        Q3CanvasLine * line = new Q3CanvasLine( getCanvas() );
         line -> setZ( 0 );
         line -> setPen( pen );
         line -> setVisible( true );
@@ -650,10 +659,12 @@ void LinePath::growList(LineList &list, int by) {
 }
 
 void LinePath::createHeadLines() {
+    qDeleteAll( m_HeadList.begin(), m_HeadList.end() );
     m_HeadList.clear();
-    QCanvas * canvas = getCanvas();
+    Q3Canvas * canvas = getCanvas();
     switch( getAssocType() ) {
     case Uml::at_Activity:
+    case Uml::at_Exception:
     case Uml::at_State:
     case Uml::at_Dependency:
     case Uml::at_UniAssociation:
@@ -664,7 +675,7 @@ void LinePath::createHeadLines() {
     case Uml::at_Generalization:
     case Uml::at_Realization:
         growList(m_HeadList, 3);
-        m_pClearPoly = new QCanvasPolygon( canvas );
+        m_pClearPoly = new Q3CanvasPolygon( canvas );
         m_pClearPoly -> setVisible( true );
         m_pClearPoly -> setBrush( QBrush( Qt::white ) );
         m_pClearPoly -> setZ( -1 );
@@ -673,7 +684,7 @@ void LinePath::createHeadLines() {
     case Uml::at_Composition:
     case Uml::at_Aggregation:
         growList(m_HeadList, 4);
-        m_pClearPoly = new QCanvasPolygon( canvas );
+        m_pClearPoly = new Q3CanvasPolygon( canvas );
         m_pClearPoly -> setVisible( true );
         if( getAssocType() == Uml::at_Aggregation )
             m_pClearPoly -> setBrush( QBrush( Qt::white ) );
@@ -745,6 +756,7 @@ void LinePath::calculateParallelLine() {
 }
 
 void LinePath::setupParallelLine() {
+    qDeleteAll( m_ParallelList.begin(), m_ParallelList.end() );
     m_ParallelList.clear();
     growList(m_ParallelList, 3);
     m_bParallelLineCreated = true;
@@ -753,7 +765,7 @@ void LinePath::setupParallelLine() {
 void LinePath::updateParallelLine() {
     if( !m_bParallelLineCreated )
         return;
-    QCanvasLine * line = 0;
+    Q3CanvasLine * line = 0;
     QPoint common = m_ParallelLines.at( 0 );
     QPoint p = m_ParallelLines.at( 1 );
     line = m_ParallelList.at( 0 );
@@ -768,7 +780,7 @@ void LinePath::updateParallelLine() {
     line -> setPoints( common.x(), common.y(), p.x(), p.y() );
 }
 
-bool LinePath::operator==( LinePath & rhs ) {
+bool LinePath::operator==( const LinePath & rhs ) {
     if( this -> m_LineList.count() != rhs.m_LineList.count() )
         return false;
 
@@ -780,14 +792,12 @@ bool LinePath::operator==( LinePath & rhs ) {
     return true;
 }
 
-LinePath & LinePath::operator=( LinePath & rhs ) {
+LinePath & LinePath::operator=( const LinePath & rhs ) {
     if( this == &rhs )
         return *this;
     //clear out the old canvas objects
-    this -> m_LineList.clear();
-    this -> m_ParallelList.clear();
-    this -> m_RectList.clear();
-    this -> m_HeadList.clear();
+    this->cleanup();
+
     int count = rhs.m_LineList.count();
     //setup start end points
     this -> setStartEndPoints( rhs.getPoint( 0 ), rhs.getPoint( count) );
@@ -800,14 +810,14 @@ LinePath & LinePath::operator=( LinePath & rhs ) {
     return *this;
 }
 
-QCanvas * LinePath::getCanvas() {
+Q3Canvas * LinePath::getCanvas() {
     if( !m_pAssociation )
         return 0;
     const UMLView * view =  m_pAssociation->getUMLView();
     return view -> canvas();
 }
 
-Uml::Association_Type LinePath::getAssocType() {
+Uml::Association_Type LinePath::getAssocType() const {
     if( m_pAssociation )
         return m_pAssociation -> getAssocType();
     return Uml::at_Association;
@@ -826,26 +836,37 @@ uint LinePath::getLineWidth() {
     if ( viewLineWidth >= 0 && viewLineWidth <= 10 )
         return viewLineWidth;
     else {
-        kWarning() << "Ignore wrong LineWidth of " << viewLineWidth
+        uWarning() << "Ignore wrong LineWidth of " << viewLineWidth
         << " in LinePath::getLineWidth" << endl;
         return 0;
     }
 }
 
 void LinePath::cleanup() {
-    if (m_pAssociation)
+    if (m_pAssociation) {
+        qDeleteAll( m_LineList.begin(), m_LineList.end() );
         m_LineList.clear();
+    }
+
+    qDeleteAll( m_HeadList.begin(), m_HeadList.end() );
     m_HeadList.clear();
+
+    qDeleteAll( m_RectList.begin(), m_RectList.end() );
     m_RectList.clear();
+
+    qDeleteAll( m_ParallelList.begin(), m_ParallelList.end() );
     m_ParallelList.clear();
 
     if( m_pClearPoly )
         delete m_pClearPoly;
     if( m_pCircle )
         delete m_pCircle;
+    if( m_pSubsetSymbol )
+        delete m_pSubsetSymbol;
     m_pCircle = 0;
     m_pClearPoly = 0;
-    m_bHeadCreated = m_bParallelLineCreated = false;
+    m_pSubsetSymbol = 0;
+    m_bHeadCreated = m_bParallelLineCreated = m_bSubsetSymbolCreated = false;
     if( m_pAssociation ) {
         UMLView * view =  (UMLView *)m_pAssociation -> parent();
         if(view) {
@@ -860,7 +881,7 @@ void LinePath::setDockRegion( Region region ) {
     m_DockRegion = region;
 }
 
-bool LinePath::hasPoints () {
+bool LinePath::hasPoints () const {
     int count = m_LineList.count();
     if (count>1)
         return true;
@@ -870,7 +891,7 @@ void LinePath::dumpPoints () {
     int count = m_LineList.count();
     for( int i = 1; i < count; i++ ) {
         QPoint point = getPoint( i );
-        kDebug()<<" * point x:"<<point.x()<<" y:"<<point.y()<<endl;
+        uDebug()<<" * point x:"<<point.x()<<" y:"<<point.y();
     }
 }
 
@@ -942,11 +963,11 @@ void LinePath::activate() {
     int count = m_LineList.count();
     if (count == 0)
         return;
-    QCanvas * canvas = getCanvas();
+    Q3Canvas * canvas = getCanvas();
     if (canvas == NULL)
         return;
     for (int i = 0; i < count ; i++) {
-        QCanvasLine *line = m_LineList.at(i);
+        Q3CanvasLine *line = m_LineList.at(i);
         line -> setCanvas( canvas );
         line -> setPen( getPen() );
     }
@@ -954,4 +975,80 @@ void LinePath::activate() {
 
 
 
+void LinePath::createSubsetSymbol() {
+    if ( m_LineList.count() < 1 ) {
+        return;
+    }
+
+    switch( getAssocType() ) {
+       case Uml::at_Child2Category:
+           m_pSubsetSymbol = new SubsetSymbol(getCanvas());
+           m_pSubsetSymbol->setPen( QPen( getLineColor(), getLineWidth() ) );
+           updateSubsetSymbol();
+           m_pSubsetSymbol->show();
+           break;
+       default:
+           break;
+    }
+    m_bSubsetSymbolCreated = true;
+}
+
+
+void LinePath::updateSubsetSymbol() {
+    if ( m_LineList.count() < 1 ) {
+        return;
+    }
+    Q3CanvasLine* firstLine = m_LineList.first();
+    QPoint startPoint = firstLine->startPoint();
+    QPoint endPoint = firstLine->endPoint();
+    QPoint centrePoint;
+    centrePoint.setX( ( startPoint.x() + endPoint.x() )/2 );
+    centrePoint.setY( ( startPoint.y() + endPoint.y() )/2 );
+
+    if ( m_pSubsetSymbol ) {
+
+        double xDiff = endPoint.x() - startPoint.x();
+        double yDiff = endPoint.y() - startPoint.y();
+
+        int inclination;
+        if ( xDiff == 0 ) {
+            if ( yDiff > 0 )
+                inclination = 90;
+            else // yDiff < 0
+                inclination = 270;
+        } else {
+           inclination = (int)(atan( yDiff/xDiff )*180/3.14159) ;
+           // convert to 360 degree scale
+           if (  xDiff < 0 ) {
+               inclination = 180 + inclination ;
+           } else if ( xDiff > 0 && yDiff < 0 ) {
+               inclination = 360 +  inclination;
+           }
+        }
+
+        m_pSubsetSymbol->setInclination(inclination);
+        m_pSubsetSymbol->setX(centrePoint.x());
+        m_pSubsetSymbol->setY(centrePoint.y());
+    }
+}
+
+LinePath::SubsetSymbol::SubsetSymbol(Q3Canvas* canvas)
+    : Q3CanvasEllipse(canvas) {
+    inclination = 0;
+}
+
+
+
+void LinePath::SubsetSymbol::drawShape(QPainter& p) {
+    p.translate(QPoint( ( int )x(), ( int )y() ) );
+    p.rotate( inclination );
+    int width = 30, height = 20;
+    int startAngle = 90, endAngle = 180;
+    p.drawArc( 0 ,-height/2, width, height, startAngle*16, endAngle*16 );
+    // revert back
+    p.rotate( -inclination );
+    p.translate( QPoint( ( int )-x(), ( int )-y() ) );
+
+}
+
 #include "linepath.moc"
