@@ -5,7 +5,7 @@
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2002-2006                                               *
+ *   copyright (C) 2002-2008                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
@@ -40,7 +40,7 @@ StateWidget::StateWidget(UMLView * view, StateType stateType, Uml::IDType id)
 StateWidget::~StateWidget() {}
 
 void StateWidget::draw(QPainter & p, int offsetX, int offsetY) {
-    UMLWidget::setPen(p);
+    setPenFromSettings(p);
     const int w = width();
     const int h = height();
     switch (m_StateType)
@@ -62,7 +62,7 @@ void StateWidget::draw(QPainter & p, int offsetX, int offsetY) {
                 p.drawText(offsetX + STATE_MARGIN, offsetY + textStartY,
                            w - STATE_MARGIN * 2, fontHeight,
                            Qt::AlignCenter, getName());
-                UMLWidget::setPen(p);
+                setPenFromSettings(p);
             } else {
                 p.drawRoundRect(offsetX, offsetY, w, h, (h*40)/w, (w*40)/h);
                 textStartY = offsetY + STATE_MARGIN;
@@ -74,7 +74,7 @@ void StateWidget::draw(QPainter & p, int offsetX, int offsetY) {
                            fontHeight, Qt::AlignCenter, getName());
                 font.setBold( false );
                 p.setFont( font );
-                UMLWidget::setPen(p);
+                setPenFromSettings(p);
                 int linePosY = textStartY + fontHeight;
 
                 QStringList::Iterator end(m_Activities.end());
@@ -84,7 +84,7 @@ void StateWidget::draw(QPainter & p, int offsetX, int offsetY) {
                     p.setPen(Qt::black);
                     p.drawText(offsetX + STATE_MARGIN, textStartY, w - STATE_MARGIN * 2 - 1,
                                fontHeight, Qt::AlignCenter, *it);
-                    UMLWidget::setPen(p);
+                    setPenFromSettings(p);
                     linePosY += fontHeight;
                 }//end for
             }//end else
@@ -103,7 +103,7 @@ void StateWidget::draw(QPainter & p, int offsetX, int offsetY) {
         p.drawEllipse( offsetX + 3, offsetY + 3, w - 6, h - 6 );
         break;
     default:
-        kWarning() << "Unknown state type:" << m_StateType << endl;
+        uWarning() << "Unknown state type:" << m_StateType;
         break;
     }
     if(m_bSelected)
@@ -155,33 +155,31 @@ void StateWidget::setStateType( StateType stateType ) {
     m_StateType = stateType;
 }
 
-void StateWidget::slotMenuSelection(int sel) {
-    bool done = false;
+void StateWidget::slotMenuSelection(QAction* action) {
     bool ok = false;
     QString name = getName();
 
+    ListPopupMenu::Menu_Type sel = m_pMenu->getMenuType(action);
     switch( sel ) {
     case ListPopupMenu::mt_Rename:
         name = KInputDialog::getText( i18n("Enter State Name"), i18n("Enter the name of the new state:"), getName(), &ok );
         if( ok && name.length() > 0 )
             setName( name );
-        done = true;
         break;
 
     case ListPopupMenu::mt_Properties:
         showProperties();
-        done = true;
         break;
+
     case ListPopupMenu::mt_New_Activity:
         name = KInputDialog::getText( i18n("Enter Activity"), i18n("Enter the name of the new activity:"), i18n("new activity"), &ok );
         if( ok && name.length() > 0 )
             addActivity( name );
-        done = true;
         break;
-    }
 
-    if( !done )
-        UMLWidget::slotMenuSelection( sel );
+    default:
+        UMLWidget::slotMenuSelection(action);
+    }
 }
 
 bool StateWidget::addActivity( const QString &activity ) {
@@ -191,10 +189,8 @@ bool StateWidget::addActivity( const QString &activity ) {
 }
 
 bool StateWidget::removeActivity( const QString &activity ) {
-    int index = - 1;
-    if( ( index = m_Activities.findIndex( activity ) ) == -1 )
+    if( m_Activities.removeAll( activity ) == 0 )
         return false;
-    m_Activities.remove( m_Activities.at( index ) );
     updateComponentSize();
     return true;
 }
@@ -210,7 +206,7 @@ QStringList & StateWidget::getActivityList() {
 
 bool StateWidget::renameActivity( const QString &activity, const QString &newName ) {
     int index = - 1;
-    if( ( index = m_Activities.findIndex( activity ) ) == -1 )
+    if( ( index = m_Activities.indexOf( activity ) ) == -1 )
         return false;
     m_Activities[ index ] = newName;
     return true;
