@@ -1,11 +1,10 @@
 /***************************************************************************
- *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
  *   the Free Software Foundation; either version 2 of the License, or     *
  *   (at your option) any later version.                                   *
  *                                                                         *
- *   copyright (C) 2006-2007                                               *
+ *   copyright (C) 2006-2008                                               *
  *   Umbrello UML Modeller Authors <uml-devel@uml.sf.net>                  *
  ***************************************************************************/
 
@@ -16,20 +15,21 @@
 #include <math.h>
 
 // include files for Qt
-#include <qstringlist.h>
-#include <qrect.h>
-#include <qimage.h>
-#include <qpicture.h>
-#include <qpainter.h>
-#include <qprinter.h>
-#include <qdir.h>
-#include <qregexp.h>
+#include <QtGui/QApplication>
+#include <QtGui/QImageWriter>
+#include <QtCore/QRect>
+#include <QtGui/QImage>
+#include <QtGui/QPicture>
+#include <QtGui/QPainter>
+#include <QtGui/QPrinter>
+#include <QtCore/QDir>
+#include <QtCore/QRegExp>
+#include <QtSvg/QSvgGenerator>
 
 // kde include files
 #include <kdebug.h>
 #include <klocale.h>
-#include <ktempfile.h>
-#include <kapplication.h>
+#include <ktemporaryfile.h>
 #include <kio/netaccess.h>
 
 // application specific includes
@@ -42,26 +42,30 @@
 static QStringList supportedImageTypesList;
 static QStringList supportedMimeTypesList;
 
-QStringList UMLViewImageExporterModel::supportedImageTypes() {
+QStringList UMLViewImageExporterModel::supportedImageTypes()
+{
     if (!supportedImageTypesList.size()) {
         // specific supported formats
         supportedImageTypesList << "eps";
         supportedImageTypesList << "svg";
 
         // QT supported formats
-        QStrList qImageFormats = QImage::outputFormats();
-        for (const char* format = qImageFormats.first(); format; format = qImageFormats.next()) {
-            supportedImageTypesList << QString(format).lower();
+        QList<QByteArray> qImageFormats = QImageWriter::supportedImageFormats();
+        QList<QByteArray>::const_iterator it, it_end;
+        it = qImageFormats.begin(); it_end = qImageFormats.end();
+        for (; it != it_end; ++it) {
+            supportedImageTypesList << QString(*it).toLower();
         }
     }
 
     return supportedImageTypesList;
 }
 
-QStringList UMLViewImageExporterModel::supportedMimeTypes() {
+QStringList UMLViewImageExporterModel::supportedMimeTypes()
+{
     if (!supportedMimeTypesList.size()) {
-        QStringList imageTypes = UMLViewImageExporterModel::supportedImageTypes();
-        for(QStringList::Iterator it = imageTypes.begin(); it != imageTypes.end(); ++it ) {
+        const QStringList imageTypes = UMLViewImageExporterModel::supportedImageTypes();
+        for (QStringList::ConstIterator it = imageTypes.begin(); it != imageTypes.end(); ++it) {
             QString mimeType = imageTypeToMimeType(*it);
             if (!mimeType.isNull())
                 supportedMimeTypesList.append(mimeType);
@@ -71,44 +75,47 @@ QStringList UMLViewImageExporterModel::supportedMimeTypes() {
     return supportedMimeTypesList;
 }
 
-QString UMLViewImageExporterModel::imageTypeToMimeType(const QString& imageType) {
-    const QString imgType = imageType.lower();
-    if (QString("bmp") == imgType) return "image/x-bmp";
+QString UMLViewImageExporterModel::imageTypeToMimeType(const QString& imageType)
+{
+    const QString imgType = imageType.toLower();
+    if (QString("bmp") == imgType) return "image/bmp";
     if (QString("jpeg") == imgType) return "image/jpeg";
     if (QString("pbm") == imgType) return "image/x-portable-bitmap";
-    if (QString("pgm") == imgType) return "image/x-portable-greymap";
+    if (QString("pgm") == imgType) return "image/x-portable-graymap";
     if (QString("png") == imgType) return "image/png";
     if (QString("ppm") == imgType) return "image/x-portable-pixmap";
-    if (QString("xbm") == imgType) return "image/x-xbm";
-    if (QString("xpm") == imgType) return "image/x-xpm";
+    if (QString("xbm") == imgType) return "image/x-xbitmap";
+    if (QString("xpm") == imgType) return "image/x-xpixmap";
     if (QString("eps") == imgType) return "image/x-eps";
     if (QString("svg") == imgType) return "image/svg+xml";
-    return QString::null;
+    return QString();
 }
 
-QString UMLViewImageExporterModel::mimeTypeToImageType(const QString& mimeType) {
-    if (QString("image/x-bmp") == mimeType) return "bmp";
+QString UMLViewImageExporterModel::mimeTypeToImageType(const QString& mimeType)
+{
+    if (QString("image/bmp") == mimeType) return "bmp";
     if (QString("image/jpeg") == mimeType) return "jpeg";
     if (QString("image/x-portable-bitmap") == mimeType) return "pbm";
-    if (QString("image/x-portable-greymap") == mimeType) return "pgm";
+    if (QString("image/x-portable-graymap") == mimeType) return "pgm";
     if (QString("image/png") == mimeType) return "png";
     if (QString("image/x-portable-pixmap") == mimeType) return "ppm";
-    if (QString("image/x-xbm") == mimeType) return "xbm";
-    if (QString("image/x-xpm") == mimeType) return "xpm";
+    if (QString("image/x-xbitmap") == mimeType) return "xbm";
+    if (QString("image/x-xpixmap") == mimeType) return "xpm";
     if (QString("image/x-eps") == mimeType) return "eps";
     if (QString("image/svg+xml") == mimeType) return "svg";
-    return QString::null;
+    return QString();
 }
 
-QStringList UMLViewImageExporterModel::exportAllViews(const QString &imageType, const KURL &directory, bool useFolders) const {
+QStringList UMLViewImageExporterModel::exportAllViews(const QString &imageType, const KUrl &directory, bool useFolders) const
+{
     UMLApp *app = UMLApp::app();
 
     // contains all the error messages returned by exportView calls
     QStringList errors;
 
     UMLViewList views = app->getDocument()->getViewIterator();
-    for(UMLView *view = views.first(); view; view = views.next()) {
-        KURL url = directory;
+    foreach (UMLView *view , views ) {
+        KUrl url = directory;
         url.addPath(getDiagramFileName(view, imageType, useFolders));
 
         QString returnString = exportView(view, imageType, url);
@@ -120,57 +127,55 @@ QStringList UMLViewImageExporterModel::exportAllViews(const QString &imageType,
     return errors;
 }
 
-QString UMLViewImageExporterModel::exportView(UMLView* view, const QString &imageType, const KURL &url) const {
+QString UMLViewImageExporterModel::exportView(UMLView* view, const QString &imageType, const KUrl &url) const
+{
     // create the needed directories
     if (!prepareDirectory(url)) {
-        return i18n("Can not create directory: %1").arg(url.directory());
+        return i18n("Can not create directory: %1", url.directory());
     }
 
     // The fileName will be used when exporting the image. If the url isn't local,
     // the fileName is the name of a temporal local file to export the image to, and then
     // upload it to its destiny
     QString fileName;
-    // tmpFile needs to be unlinked before exiting the method!!!
-    KTempFile tmpFile;
+    KTemporaryFile tmpFile;
     if (url.isLocalFile()) {
         fileName = url.path();
     } else {
-        fileName = tmpFile.name();
+        tmpFile.open();
+        fileName = tmpFile.fileName();
     }
 
     // check that the diagram isn't empty
     QRect rect = view->getDiagramRect();
     if (rect.isEmpty()) {
-        tmpFile.unlink();
         return i18n("Can not save an empty diagram");
     }
 
     // exporting the view to the file
     if (!exportViewTo(view, imageType, fileName)) {
-        tmpFile.unlink();
-        return i18n("A problem occured while saving diagram in %1").arg(fileName);
+        return i18n("A problem occurred while saving diagram in %1", fileName);
     }
 
     // if the file wasn't local, upload the temp file to the target
     if (!url.isLocalFile()) {
-        if (!KIO::NetAccess::upload(tmpFile.name(), url, UMLApp::app())) {
-            tmpFile.unlink();
-            return i18n("There was a problem saving file: %1").arg(url.path());
+        if (!KIO::NetAccess::upload(tmpFile.fileName(), url, UMLApp::app())) {
+            return i18n("There was a problem saving file: %1", url.path());
         }
     } //!isLocalFile
 
-    tmpFile.unlink();
-    return QString::null;
+    return QString();
 }
 
-QString UMLViewImageExporterModel::getDiagramFileName(UMLView *view, const QString &imageType, bool useFolders /* = false */) const {
-    QString name = view->getName() + '.' + imageType.lower();
+QString UMLViewImageExporterModel::getDiagramFileName(UMLView *view, const QString &imageType, bool useFolders /* = false */) const
+{
+    QString name = view->getName() + '.' + imageType.toLower();
 
     if (!useFolders) {
         return name;
     }
 
-    kapp->processEvents();
+    qApp->processEvents();
     UMLListView *listView = UMLApp::app()->getListView();
     UMLListViewItem* listViewItem = listView->findItem(view->getID());
     // skip the name of the first item because it's the View
@@ -188,17 +193,18 @@ QString UMLViewImageExporterModel::getDiagramFileName(UMLView *view, const QStri
     return name;
 }
 
-bool UMLViewImageExporterModel::prepareDirectory(const KURL &url) const {
-    // the KURL is copied to get protocol, user and so on and then the path is cleaned
-    KURL directory = url;
+bool UMLViewImageExporterModel::prepareDirectory(const KUrl &url) const
+{
+    // the KUrl is copied to get protocol, user and so on and then the path is cleaned
+    KUrl directory = url;
     directory.setPath("");
 
     // creates the directory and any needed parent directories
-    QStringList dirs = QStringList::split(QDir::separator(), url.directory());
-    for (QStringList::ConstIterator it = dirs.begin() ; it != dirs.end(); ++it ) {
+    QStringList dirs = url.directory().split(QDir::separator(), QString::SkipEmptyParts );
+    for (QStringList::ConstIterator it = dirs.begin() ; it != dirs.end(); ++it) {
         directory.addPath(*it);
 
-        if (!KIO::NetAccess::exists(directory, true, UMLApp::app())) {
+        if (!KIO::NetAccess::exists(directory, KIO::NetAccess::SourceSide, UMLApp::app())) {
 
             if (!KIO::NetAccess::mkdir(directory, UMLApp::app())) {
                 return false;
@@ -209,7 +215,8 @@ bool UMLViewImageExporterModel::prepareDirectory(const KURL &url) const {
     return true;
 }
 
-bool UMLViewImageExporterModel::exportViewTo(UMLView* view, const QString &imageType, const QString &fileName) const {
+bool UMLViewImageExporterModel::exportViewTo(UMLView* view, const QString &imageType, const QString &fileName) const
+{
     // remove 'blue squares' from exported picture.
     view->clearSelected();
 
@@ -231,7 +238,8 @@ bool UMLViewImageExporterModel::exportViewTo(UMLView* view, const QString &image
     return true;
 }
 
-bool UMLViewImageExporterModel::exportViewToEps(UMLView* view, const QString &fileName, bool isEPS) const {
+bool UMLViewImageExporterModel::exportViewToEps(UMLView* view, const QString &fileName, bool isEPS) const
+{
     bool exportSuccessful = true;
 
     // print the image to a normal postscript file,
@@ -248,7 +256,6 @@ bool UMLViewImageExporterModel::exportViewToEps(UMLView* view, const QString &fi
     } else {
         printer = new QPrinter(QPrinter::ScreenResolution);
     }
-    printer->setOutputToFile(true);
     printer->setOutputFileName(fileName);
     printer->setColorMode(QPrinter::Color);
 
@@ -282,30 +289,29 @@ bool UMLViewImageExporterModel::exportViewToEps(UMLView* view, const QString &fi
     return exportSuccessful;
 }
 
-bool UMLViewImageExporterModel::fixEPS(const QString &fileName, const QRect& rect) const {
+bool UMLViewImageExporterModel::fixEPS(const QString &fileName, const QRect& rect) const
+{
     // now open the file and make a correct eps out of it
     QFile epsfile(fileName);
-    if (! epsfile.open(IO_ReadOnly)) {
+    if (! epsfile.open(QIODevice::ReadOnly)) {
         return false;
     }
     // read
     QTextStream ts(&epsfile);
-    QString fileContent = ts.read();
+    QString fileContent = ts.readAll();
     epsfile.close();
 
     // read information
     QRegExp rx("%%BoundingBox:\\s*(-?[\\d\\.:]+)\\s*(-?[\\d\\.:]+)\\s*(-?[\\d\\.:]+)\\s*(-?[\\d\\.:]+)");
-    const int pos = rx.search(fileContent);
+    const int pos = rx.indexIn(fileContent);
     if (pos < 0) {
-        kError() << "UMLViewImageExporterModel::fixEPS(" << fileName
-                  << "): cannot find %%BoundingBox" << endl;
+        uError() << fileName << ": cannot find %%BoundingBox";
         return false;
     }
 
     // write new content to file
-    if (! epsfile.open(IO_WriteOnly | IO_Truncate)) {
-        kError() << "UMLViewImageExporterModel::fixEPS(" << fileName
-                  << "): cannot open file for writing" << endl;
+    if (! epsfile.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
+        uError() << fileName << ": cannot open file for writing";
         return false;
     }
 
@@ -328,39 +334,44 @@ bool UMLViewImageExporterModel::fixEPS(const QString &fileName, const QRect& rec
     return true;
 }
 
-bool UMLViewImageExporterModel::exportViewToSvg(UMLView* view, const QString &fileName) const {
-    bool exportSuccesful;
+bool UMLViewImageExporterModel::exportViewToSvg(UMLView* view, const QString &fileName) const
+{
+    bool exportSuccessful;
+    QRect rect = view->getDiagramRect();
 
-    QPicture* diagram = new QPicture();
-
-    // do not call printer.setup(); because we want no user
-    // interaction here
-    QPainter* painter = new QPainter();
-    painter->begin( diagram );
+    QSvgGenerator generator;
+    generator.setFileName(fileName);
+    generator.setSize(rect.size());
+    QPainter painter(&generator);
 
     // make sure the widget sizes will be according to the
     // actually used printer font, important for getDiagramRect()
     // and the actual painting
-    view->forceUpdateWidgetFontMetrics(painter);
+    view->forceUpdateWidgetFontMetrics(&painter);
 
-    QRect rect = view->getDiagramRect();
-    painter->translate(-rect.x(),-rect.y());
-    view->getDiagram(rect,*painter);
-    painter->end();
-    exportSuccesful = diagram->save(fileName, QString("SVG").ascii());
 
-    // delete painter and printer before we try to open and fix the file
-    delete painter;
-    delete diagram;
+    painter.translate(-rect.x(),-rect.y());
+    view->getDiagram(rect, painter);
+    painter.end();
+
+    //FIXME: Determine the status of svg generation.
+    exportSuccessful = true;
+
     // next painting will most probably be to a different device (i.e. the screen)
     view->forceUpdateWidgetFontMetrics(0);
 
-    return exportSuccesful;
+    uDebug() << "saving to file " << fileName << " successful=" << exportSuccessful;
+    return exportSuccessful;
 }
 
-bool UMLViewImageExporterModel::exportViewToPixmap(UMLView* view, const QString &imageType, const QString &fileName) const {
+bool UMLViewImageExporterModel::exportViewToPixmap(UMLView* view, const QString &imageType, const QString &fileName) const
+{
+    bool exportSuccessful;
     QRect rect = view->getDiagramRect();
     QPixmap diagram(rect.width(), rect.height());
     view->getDiagram(rect, diagram);
-    return diagram.save(fileName, imageType.upper().ascii());
+    exportSuccessful = diagram.save(fileName, qPrintable(imageType.toUpper()));
+
+    uDebug() << "saving to file " << fileName << " , imageType=" << imageType << " successful=" << exportSuccessful;
+    return exportSuccessful;
 }
